/*******************************************************************************
 * Copyright 2019 Graz University of Technology
 * MOA ZS has been developed in a cooperation between EGIZ
 * and Graz University of Technology.
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.gv.egiz.moazs.config;

import at.gv.egiz.moazs.preprocess.MzsDeliveryRequestValidator;
import at.gv.egiz.moazs.preprocess.*;
import at.gv.zustellung.app2mzs.xsd.ConfigType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.Map;

import static at.gv.egiz.moazs.preprocess.ConfigProfileGenerator.configProfileGeneratorBuilder;

/**
 * @author Christof Rabensteiner
 *
 */
@Configuration
public class PreprocessConfig {

    @Bean
    @Autowired
    public Map<String, ConfigType> deliveryRequestConfigs(ConfigProfileGenerator generator) {
        return generator.generate();
    }

    @Bean
    @Autowired
    public ConfigProfileGenerator configProfileGenerator(
            SpringPropertiesFacade properties,
            MzsDeliveryRequestValidator validator,
            ConfigUtil util,
            @Value("${verify-completeness-of-default-delivery-request-configuration:true}")
                    boolean verifyCompletenessOfDefaultConfiguration) {

        return configProfileGeneratorBuilder()
                .withProperties(properties)
                .withConfigUtil(util)
                .withVerifyCompletenessOfDefaultConfiguration(verifyCompletenessOfDefaultConfiguration)
                .withValidator(validator)
                .build();
    }

}
